/*
 * Control file for CMPE490, G9
 *
 */

#ifdef SEMIHOSTING
#include <stdio.h>
#endif


#include "etc.h"


#define numPlayStates 27

extern void wake_up_handler(void);
extern void irqasmhandler(void);
extern void PIOirqhandler(void);

WaitDesc wait_desc = { &TC0_DESC, 0, 0, WAIT_DELAY, wake_up_handler } ;
char playState;
char imgNum;


//This is the interrupt handler
//for the pushbutton.
//Serves to iterate the state machine
void pioHandler(void)
{

    playState++;
    playState%=numPlayStates;

}

//Interrupt handler for the
//Timer channel; serves to modulate
//the laser
void intHandler(void)
{
    //Dummy variable to hold the result of TC_SR read
    u_short x;
    
    //Increment the cycle counter
    cycles++;
    if(cycles+1==cycles)
	cycles=0;

    //Write PIO pin; see at91_pio_write()
    if(pioWriter==PIO_CLEAR_OUT)
	(&PIOB_DESC)->pio_base->PIO_CODR=MODPIN;
    else
	(&PIOB_DESC)->pio_base->PIO_SODR=MODPIN;
	

    //read the timer channel SR to clear
    //interrupt status
    x=(&TC2_DESC)->tc_base->TC_SR;

}

int main(void)
{
//Variables
    u_int tcreg[4]; 
    pioWriter=0;
    imgNum=0;
    cycles=0;
    playState=0;

    //Timer controls
    //MCK=32 MHz; MCK/2=16 MHz
    //533 MCK/2 Ticks req'd before change.
    //Set RC=266 for 30kpps
    //each cycle tick is 33.3 us
    tcreg[RA]=0;
    tcreg[RB]=0;
    tcreg[RC]=533;

    EnableInterrupt(); //Enables interrupts if slingshot has disabled them
    //This function is defined in myASM.arm

    //Open the timers
    at91_tc_open(&TC1_DESC,TC_WAVE|TC_ACPC_TOGGLE_OUTPUT|TC_CLKS_MCK2|TC_CPCTRG,TRUE,FALSE);
    at91_tc_open(&TC2_DESC,TC_WAVE|TC_ACPC_TOGGLE_OUTPUT|TC_CLKS_MCK2|TC_CPCTRG,TRUE,FALSE);

    //Enable Timer2 Interrupt with priority 6
    (&TC2_DESC)->tc_base->TC_IER|=(1<<4);
    at91_irq_open(8,6,AIC_SRCTYPE_INT_LEVEL_SENSITIVE,&irqasmhandler);

    //Write the TC registers specified above
    at91_tc_write(&TC1_DESC,tcreg);
    at91_tc_write(&TC2_DESC,tcreg);

    //Open the modulation PIO pin
    at91_pio_open(&PIOB_DESC,MODPIN,PIO_OUTPUT); //Enable the laser modulation pin

    //Open the interrupt attached to pushbutton 2 (SW2)
    at91_extirq_open(&IRQ0_DESC,7,AIC_SRCTYPE_EXT_NEGATIVE_EDGE,&PIOirqhandler);
	

	
	//DACs
    at91_dac_open(&DAC0_DESC,DAC_TTRGEN_EN|DAC_10_BIT_RES|DAC_TRG_TIOA1);
    at91_dac_open(&DAC1_DESC,DAC_TTRGEN_EN|DAC_10_BIT_RES|DAC_TRG_TIOA1);
    //at91_dac_open(&DAC0_DESC,DAC_TTRGEN_DIS|DAC_10_BIT_RES);
    //at91_dac_open(&DAC1_DESC,DAC_TTRGEN_DIS|DAC_10_BIT_RES);


    //Start the timers
    at91_tc_trig_cmd(&TC1_DESC,TC_TRIG_CHANNEL); //Start the timer
    at91_tc_trig_cmd(&TC2_DESC,TC_TRIG_CHANNEL); //Start the timer
	
    dac0op=0;
    dac1op=0;



    //This is the state machine that controls image output
    //Images are defined in etc.c
    for(;;)
    {
	while(playState==0)
	{
	    slowPt(VERT, 40, &playState);
	    
	    if(playState==5)
		slowPt(HORZ,40,&playState);
	}
	while(playState==1)
	    blank();
	
	while(playState==2)
	{
	    peace(&playState);
	}
	while(playState==3)
	{
	    peaceDelay(&playState);
	}
	
	while(playState==4)
	{
	    peace(&playState);
	}
	while(playState==5)
	    blank();
	
	while(playState==6)
	{
	    slowPt(VERT,100, &playState);
	}
	while(playState==7)
	{
	    box(&playState);
	}
	while(playState==8)
	    blank();
	
	while(playState==9)
	{
	    slowPt(VERT, 40, &playState);
	    
	    if(playState==9)
		slowPt(HORZ,40,&playState);
	}
	while(playState==10)
	    blank();

	while(playState==11)
	{
	    blink(512,512,&playState);
	}
	while(playState==12)
	{
	    //pulsed line
	    dac1op=400;
	    scan(&dac0op,10, 1, 10, &playState);
	}
	while(playState==13)
	    blank();
	while(playState==14)
	{
	    tek(&playState);
	    
	}
	while(playState==15)
	    coke(&playState);
	
	while(playState==16)
	{
	    dac1op=700;
	    scan2(&dac0op,362,662,5,&playState);
	}
	while(playState==17)
	{
	    dac1op=300;
	    scan2(&dac0op,212,812,5,&playState);
	}
	while(playState==18)
	    fan(HORZ,50,&playState);

	while(playState==19)
	{
	    dac1op=300;
	    scan2(&dac0op,212,812,5,&playState);
	}

	while(playState==20)
	{
	    dac1op=700;
	    scan2(&dac0op,362,662,5,&playState);
	}

	while(playState==21)
	{
	    //2vafans
	    dac0op=312;
	    scan2(&dac1op,312,712,5,&playState);
	    
	    pioWriter=PIO_LOW;
	    writeDACs();
	    pause(200,&playState);
	    
	    dac0op=712;
	    scan2(&dac1op,312,712,5,&playState);

	    pioWriter=PIO_LOW;
	    writeDACs();
	    pause(200,&playState);
	}

	while(playState==22)
	{
	    //2vafans
	    dac0op=212;
	    scan2(&dac1op,212,812,5,&playState);
	    
	    pioWriter=PIO_LOW;
	    writeDACs();
	    pause(200,&playState);
	    
	    dac0op=812;
	    scan2(&dac1op,212,812,5,&playState);

	    pioWriter=PIO_LOW;
	    writeDACs();
	    pause(200,&playState);
	}
	while(playState==23)
	{
	    circle(&playState);
	}	

	while(playState==24)
	    android(&playState);
	
	while(playState==25)
	{
	    //vscanning fan
	    u_short xx=1;
	    for(xx=200;xx<=800 && playState==5;xx++)
		fan(0,xx,&playState);
	    for(;xx >=200 && playState==5;xx--)
		fan(0,xx,&playState);
	}
	while(playState==26)
	{
	    diPoint(100,100,600,600,20,0,&playState);
	}
    }

    return 0;
}
